package com.adc.da.znks.controller;

import com.adc.da.znks.entity.Admin;
import com.adc.da.znks.entity.ResponseEntity;
import com.adc.da.znks.service.AdminService;
import com.adc.da.znks.util.GenResponse;
import com.adc.da.znks.vo.AdminSearch;
import com.adc.da.znks.vo.AdminVO;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author ley
 **/
@RestController
@RequestMapping("/api")
public class AdminController {

    @Autowired
    private AdminService adminService;

    @PostMapping("/admin")
    public ResponseEntity saveOne(Admin admin) {
        try {
            int result=adminService.insertAdmin(admin);
            if(result==1){
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "保存成功",true);
            }else{
                return GenResponse.fail("数据库已有该管理员名,插入失败!");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("保存失败");
        }
    }


    @PutMapping("/admin")
    public ResponseEntity<Boolean> updateOne(Admin admin) {
        try {
            int result=adminService.updateAdmin(admin);
            if(result==1){
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "更新成功",true);
            }else{
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "更新失败",false);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("更新失败");
        }
    }


    @DeleteMapping("/admin/{adminName}")
    public ResponseEntity deleteOne(@PathVariable String adminName) {
        try {
            int result = adminService.deleteAdmin(adminName);
            if(result==1){
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "删除成功",true);
            }else{
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "删除失败",false);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("删除失败");
        }
    }


    @GetMapping("/admin/{adminName}")
    public ResponseEntity<Admin> findOne(@PathVariable String adminName) {
        try {
            Admin admin = adminService.findAdmin(adminName);
            return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                    "查询成功", admin);
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("查询失败");
        }
    }


    @GetMapping(value = "/admin/page")
    public ResponseEntity<PageInfo> findByPage(AdminSearch adminSearch, @RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "8") Integer pageSize) {
        try {
            PageHelper.startPage(pageNum, pageSize);
            List<AdminVO> adminVOS = adminService.listAdminsByCondition(adminSearch, pageNum, pageSize);
            PageInfo<AdminVO> pageInfo = new PageInfo<AdminVO>(adminVOS);
            return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                    "查询成功", pageInfo);
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("查询失败");
        }
    }


    @CrossOrigin
    @PostMapping(value = "/admin/login",consumes = {"application/x-www-form-urlencoded"})
    public ResponseEntity login(@RequestParam String adminName, @RequestParam String password) {
        try {
            if (adminService.login(adminName, password)) {
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "登录成功", true);
            } else {
                return GenResponse.fail("登录失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("登录失败");
        }
    }


}
