package com.adc.da.znks.controller;

import com.adc.da.znks.entity.ResponseEntity;
import com.adc.da.znks.entity.User;
import com.adc.da.znks.service.UserService;
import com.adc.da.znks.util.GenResponse;
import com.adc.da.znks.util.SocketUtils;
import com.adc.da.znks.util.SystemValue;
import com.adc.da.znks.util.netty.ServerHandler;
import io.netty.channel.Channel;
import io.swagger.annotations.Api;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * 命令操作activity
 * @author David
 */
@RestController
@RequestMapping("/api/instructions")
@Api(description = "| Instructions |")
public class InstructionsController {

    @Autowired
    private UserService userService;

    private static final Logger logger = LoggerFactory.getLogger(InstructionsController.class);

    @GetMapping("/loginGateway")
    public ResponseEntity<Boolean> loginGateway(String phoneNumber) {
        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            // 登录码
            byte[] login = new byte[8];
            login[0] = 0x2a;
            login[1] = 0x01;
            login[2] = 0x02;
            login[3] = 0x03;
            login[4] = 0x04;
            login[5] = 0x05;
            login[6] = 0x06;
            login[7] = 0x2a;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(login);
            String result = "1";
            if("01".equals(result)) {
                return GenResponse.success("1", "登录成功", true);
            } else if("00".equals(result)) {
                return GenResponse.fail("登录失败");
            } else {
                return GenResponse.fail("登录异常");
            }
        } else {
            return GenResponse.fail("登录异常");
        }
    }

    @GetMapping("/openLockOrLearn")
    public ResponseEntity<Boolean> openLockOrLearn(String phoneNumber, int number) {
        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[8];
            open[0] = 0x54;
            open[1] = 0x59;
            open[2] = 0x54;
            open[3] = 0x00;
            open[4] = 0x01;
            open[5] = 0x00;
            open[6] = (byte)number;
            open[7] = 0x2d;
            logger.info("设备吗学习指令发送：++++++++++++++++++++++++++++++++"+SocketUtils.bytesToHex(open,0,open.length));
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("开锁异常");
        }
    }

    @GetMapping("/securityCodeLearn")
    public ResponseEntity<Boolean> securityCodeLearn(String phoneNumber, int number) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[8];
            open[0] = 0x54;
            open[1] = 0x59;
            open[2] = 0x54;
            open[3] = 0x01;
            open[4] = 0x01;
            open[5] = 0x00;
            open[6] = (byte)number;
            open[7] = 0x01;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("安防码学习异常");
        }
    }

    /**
     * 布防
     * @param phoneNumber
     * @return
     */
    @GetMapping("/securityAdd")
    public ResponseEntity<Boolean> securityAdd(String phoneNumber) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[8];
            open[0] = 0x54;
            open[1] = 0x59;
            open[2] = 0x54 ;
            open[3] = 0x30;
            open[4] = 0x01;
            open[5] = 0x00;
            open[6] = 0x00;
            open[7] = (byte)0xF0;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("布防失败");
        }
    }

    /**
     * 撤防
     * @param phoneNumber
     * @return
     */
    @GetMapping("/securityCancel")
    public ResponseEntity<Boolean> securityCancel(String phoneNumber) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[8];
            open[0] = 0x54;
            open[1] = 0x59;
            open[2] = 0x54 ;
            open[3] = 0x30;
            open[4] = 0x01;
            open[5] = 0x00;
            open[6] = 0x00;
            open[7] = (byte)0xF1;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("撤防失败");
        }
    }

    /**
     * 进入组网
     * @param phoneNumber
     * @return
     */
    @GetMapping("/enterZigbee")
    public ResponseEntity<Boolean> enterZigbee(String phoneNumber) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[6];
            open[0] = 0x5C;
            open[1] = (byte)0xC5;
            open[2] = (byte)0xFF ;
            open[3] = 0x01;
            open[4] = 0x01;
            open[5] = 0x22;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("进入组网失败");
        }
    }

    /**
     * 退出组网
     * @param phoneNumber
     * @return
     */
    @GetMapping("/abortZigbee")
    public ResponseEntity<Boolean> abortZigbee(String phoneNumber) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[6];
            open[0] = 0x5C;
            open[1] = (byte)0xC5;
            open[2] = (byte)0xFF ;
            open[3] = 0x01;
            open[4] = 0x02;
            open[5] = 0x23;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("退出组网失败");
        }
    }

    /**
     * zigBee恢复出厂
     * @param phoneNumber
     * @return
     */
    @GetMapping("/restoreZigbee")
    public ResponseEntity<Boolean> restoreZigbee(String phoneNumber) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[6];
            open[0] = 0x5C;
            open[1] = (byte)0xC5;
            open[2] = (byte)0xFF ;
            open[3] = 0x01;
            open[4] = 0x03;
            open[5] = 0x24;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("zigBee恢复出厂失败");
        }
    }

    /**
     * wifi恢复出厂
     * @param phoneNumber
     * @return
     */
    @GetMapping("/restoreWifi")
    public ResponseEntity<Boolean> restoreWifi(String phoneNumber) {

        User user = userService.getByPhoneNumber(phoneNumber);
        if(user != null) {
            byte[] open = new byte[6];
            open[0] = 0x5C;
            open[1] = (byte)0xC5;
            open[2] = (byte)0xFF ;
            open[3] = 0x01;
            open[4] = 0x04;
            open[5] = 0x25;
            String mac = getMacByPhone(phoneNumber);
            if(mac==null){
                return GenResponse.fail("网关mac获取失败");
            }
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            System.out.println("open status : " + ch.isOpen());
            System.out.println("active status : " + ch.isActive());
            ch.writeAndFlush(open);
            return GenResponse.success();
        } else {
            return GenResponse.fail("wifi恢复出厂失败");
        }
    }

    /**
     * 验证网关是否在线
     * @param phoneNumber
     * @return
     */
    @GetMapping("/validateGateWay")
    public ResponseEntity<Boolean> validateGateWay(String phoneNumber) {

        String mac = getMacByPhone(phoneNumber);
        if(mac==null){
            return GenResponse.fail("网关mac获取失败");
        }
        if(SystemValue.channelMap.get(mac)!=null){
            Channel ch = ((Channel)SystemValue.channelMap.get(mac));
            if(ch.isActive()){
                return GenResponse.success();
            }
        }
        return GenResponse.fail("网关未连接");
    }



    public String getMacByPhone(String phone){
        List<Map<String,Object>> list = userService.queryMacByPhone(phone);
        if(list!=null){
            return list.get(0).get("mac_address")!=null?list.get(0).get("mac_address").toString():null;
        }
        return null;
    }

}
