package com.adc.da.znks.controller;


import com.adc.da.util.utils.BeanMapper;
import com.adc.da.znks.entity.Position;
import com.adc.da.znks.entity.ResponseEntity;
import com.adc.da.znks.entity.User;
import com.adc.da.znks.page.PositionPage;
import com.adc.da.znks.service.PositionService;
import com.adc.da.znks.service.UserService;
import com.adc.da.znks.util.GenResponse;
import com.adc.da.znks.vo.PositionVO;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.adc.da.base.web.BaseController;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping("/api/entity/positions")
@Api(description = "|Position|")
public class PositionController extends BaseController<Position> {


    @Autowired
    private PositionService positionService;

    @Autowired
    private UserService userService;

    @Autowired
    private BeanMapper beanMapper;

    private static final Logger logger = LoggerFactory.getLogger(PositionController.class);

    @ApiOperation(value = "|Position|分页查询")
    @GetMapping("/page")
    public ResponseEntity<PageInfo> page(@RequestParam(required = false) String telphone, @RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "8") Integer pageSize) throws Exception {
        try {
            PageHelper.startPage(pageNum, pageSize);
            List<PositionVO> positionVOS = positionService.listPositionsByPage(telphone, pageNum, pageSize);
            PageInfo positionVOPageInfo = new PageInfo(positionVOS);
            return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                    "查询位置信息成功", positionVOPageInfo);
        } catch (Exception e) {
            e.printStackTrace();
            return GenResponse.fail("查询位置信息失败");
        }
    }


    @ApiOperation(value = "|Position|新增")
    @PostMapping(value = "/add")
    public ResponseEntity<Boolean> create(PositionVO positionVO) throws Exception {
        Position position = beanMapper.map(positionVO, Position.class);
        String userTelphone = positionVO.getTelphone();
        User user = userService.getByPhoneNumber(userTelphone);
        if (user != null) {
            position.setPositionId(UUID.randomUUID().toString());
            position.setRecordTime(new Date(System.currentTimeMillis()));
            position.setUserId(user.getId());
            int result = positionService.insertSelective(position);
            if (result != 0) {
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "新增位置成功", true);
            } else {
                return GenResponse.fail("新增位置失败");
            }
        } else {
            return GenResponse.fail("用户手机号在数据库不存在");
        }
    }

//    @ApiOperation(value = "|Position|修改")
//    @PutMapping("/position")
//    public ResponseMessage<Position> update(Position position) throws Exception {
//        positionService.updateByPrimaryKeySelective(position);
//        return Result.success(position);
//    }
//
//    @ApiOperation(value = "|Position|删除")
//    @DeleteMapping("/position/{positionId}")
//    public ResponseMessage delete(@PathVariable String positionId) throws Exception {
//        positionService.deleteByPrimaryKey(positionId);
//        logger.info("delete from tb_position where positionId = {}", positionId);
//        return Result.success();
//    }

}
