package com.adc.da.znks.controller;

import com.adc.da.base.web.BaseController;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.znks.contants.ZnksConstants;
import com.adc.da.znks.dao.PositionDao;
import com.adc.da.znks.entity.ResponseEntity;
import com.adc.da.znks.entity.User;
import com.adc.da.znks.service.LockRecordService;
import com.adc.da.znks.service.UserCountService;
import com.adc.da.znks.service.UserService;
import com.adc.da.znks.util.DateTimeUtil;
import com.adc.da.znks.util.GenResponse;
import com.adc.da.znks.vo.UserCountVO;
import com.adc.da.znks.vo.UserSearch;
import com.adc.da.znks.vo.UserVO;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * @author David
 */
@RestController
@RequestMapping("/api/user")
@Api(description = "|User|")
public class UserController extends BaseController<User> {

    @Autowired
    private UserService userService;
    @Autowired
    private LockRecordService lockRecordService;

    private static final Logger log = LoggerFactory.getLogger(UserController.class);

    @Autowired
    private UserCountService userCountService;

    @ApiOperation(value = "|User|登录或者注册用户")
    @PostMapping(value = {"/loginOrRegisterUser"})
    public ResponseEntity<Boolean> loginOrRegisterUser(String phoneNumber, String password) {
        int i = userService.loginOrRegister(phoneNumber, password);
        if (i < 0) {
            return GenResponse.success(String.valueOf(HttpStatus.OK), "用户不存在", false);
        } else {
            if (i > 0) {
                return GenResponse.success(String.valueOf(HttpStatus.OK), "登陆成功", true);
            }
            return GenResponse.fail("登录失败");
        }
    }

    @ApiOperation(value = "|User|查询当前手机号是否已经注册")
    @GetMapping(value = {"/isRegister"})
    public ResponseEntity<Boolean> isRegister(String phoneNumber) {
        boolean flag = userService.getUserByPhoneNumber(phoneNumber);
        return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "登录或者注册用户成功", flag);
    }

    @ApiOperation(value = "|User|更新密码")
    @PostMapping("/updatePassword")
    public ResponseEntity<Boolean> update(String phoneNumber, String password) {
        User user = new User();
        user.setTemphone(phoneNumber);
        user.setPlainPassword(password);
        boolean flag = userService.updatePassword(user);
        if (flag) {
            return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "更新密码成功", flag);
        } else {
            return GenResponse.fail("更新密码失败");
        }
    }

    @ApiOperation(value = "|User|更新个推ClientId")
    @PostMapping("/updateClientId")
    public ResponseEntity updateClientId(String clientId, String phoneNumber) {
        userService.updateClientIdByPhoneNumber(clientId, phoneNumber);
        return GenResponse.success();
    }
    //selectLockRecord

    @ApiOperation(value = "开锁日志")
    @GetMapping("/listLockRecord")
    public ResponseMessage listLockRecord(String phone, String number) {
        return Result.success(lockRecordService.selectLockRecord(phone, number));
    }


    @ApiOperation(value = "|User|后台用户列表查询(默认第一页,每页8条)")
    @GetMapping("/page")
    public ResponseEntity<PageInfo> listUserVOs(UserSearch userSearch,
                                                @RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "8") Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<UserVO> userVOS = userService.listUserVOsByCondition(userSearch, pageNum, pageSize);
        try {
            if (CollectionUtils.isNotEmpty(userVOS)) {
                PageInfo<UserVO> pageInfo = new PageInfo<>(userVOS);
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "分页查询成功", pageInfo);
            } else {
                return GenResponse.success(String.valueOf(HttpStatus.OK.value()),
                        "没有记录可以分页", null);
            }
        } catch (Exception e) {
            return GenResponse.fail("分页查询失败");
        }
    }

    @ApiOperation(value = "更新家庭名称")
    @PostMapping("/updateHomeName")
    public ResponseEntity updateHomeName(String phoneNumber, String homeName) {
        User user = new User();
        user.setTemphone(phoneNumber);
        user.setHomeName(homeName);
        if (userService.updateHomeName(user)) {
            return GenResponse.success();
        }
        return GenResponse.fail("更新失败");
    }

    @ApiOperation(value = "获取家庭名称")
    @PostMapping("/getHomeName")
    public ResponseEntity<String> getHomeName(String phoneNumber) {
        String name = userService.selectHomeNameByPhone(phoneNumber);
        return GenResponse.success("200", "", name);
    }

    @ApiOperation(value = "|User|增加用户")
    @PostMapping(value = {"/register"})
    public ResponseEntity<Boolean> register(String phoneNumber) {
        boolean flag = userService.addUser(phoneNumber);
        return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "", flag);
    }


    @ApiOperation(value = "|User|统计用户信息")
    @PostMapping("/countUserInfo")
    public ResponseEntity<List<UserCountVO>> countUserInfo() {
        return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "统计用户信息成功",
                userCountService.getUserCountInfo());
    }

    @Autowired
    private PositionDao positionDao;

    @ApiOperation(value = "|User|统计信息")
    @PostMapping("/countInfo")
    public ResponseEntity<Map<String, Object>> countInfo() {
        Map<String, Object> map = new HashMap<>(4);
        map.put("userCount", String.valueOf(userService.getUserCount()));
        map.put("currentDayUsers", String.valueOf(userService.getCurrentDayUsers()));
        List<String> tianJinAreas = org.springframework.util.CollectionUtils.arrayToList(ZnksConstants.TIANJIN_AREAS);
        List<Map<String, String>> areaCountList = new ArrayList<>();
        for(String area : tianJinAreas) {
            int c = userService.getUserCountInTianJinPerArea(area);
            Map<String, String> map1 = new HashMap<>();
            map1.put(area, String.valueOf(c));
            areaCountList.add(map1);
        }
        map.put("areaCountList", areaCountList);

        Date currentTime = new Date(System.currentTimeMillis());
        String currentDateStr = com.adc.da.util.utils.DateUtils.dateToString(currentTime,
                "yyyy-MM-dd");

        List<String> timeCountList = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            String startTime = currentDateStr + " " + i + ":00:00";
            String endTime;
            if(i == 23) {
                endTime = currentDateStr + " 23:59:59";
            } else {
                endTime = currentDateStr + " " + (i + 1) + ":00:00";
            }
            timeCountList.add(String.valueOf(positionDao.getUserCountByPerHour(startTime, endTime)));
        }

        map.put("timeCountList", timeCountList);

        return GenResponse.success(String.valueOf(HttpStatus.OK.value()), "统计用户信息成功", map);
    }

}
