package com.adc.da.znks.service;

import com.adc.da.util.utils.UUID;
import com.adc.da.znks.dao.AdminDao;
import com.adc.da.znks.entity.Admin;
import com.adc.da.znks.util.Base64Utils;
import com.adc.da.znks.vo.AdminSearch;
import com.adc.da.znks.vo.AdminVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.List;

/**
 * @author ley
 **/
@Service
@Transactional(value = "transactionManager", propagation = Propagation.REQUIRED, rollbackFor = Throwable.class)
public class AdminService {

    @Autowired(required = false)
    private AdminDao adminDao;


    public List<AdminVO> listAdminsByCondition(AdminSearch adminSearch, Integer pageNum, Integer pageSize) {
        return adminDao.listAdminsByCondition(adminSearch, pageNum, pageSize);
    }


    public int insertAdmin(Admin admin) {
        Admin findAdmin = adminDao.findAdminByAdminName(admin.getName());
        if (findAdmin == null) {
            admin.setAdminId(UUID.randomUUID());
            admin.setCreateTime(new Date(System.currentTimeMillis()));
            if (StringUtils.hasText(admin.getPlainPassword())) {
                admin.setPassword(Base64Utils.encode(admin.getPlainPassword().getBytes(Charset.defaultCharset())));
            }
            return adminDao.insertAdmin(admin);
        } else {
            //数据库已有该管理员名,不允许插入
            return -1;
        }

    }


    public int deleteAdmin(String adminId) {
        return adminDao.deleteAdmin(adminId);
    }


    public int updateAdmin(Admin admin) {
        Admin findAdmin = adminDao.findAdminByAdminName(admin.getName());
        if (findAdmin != null) {
            if (StringUtils.hasText(admin.getPlainPassword())) {
                findAdmin.setPassword(Base64Utils.encode(admin.getPlainPassword().getBytes(Charset.defaultCharset())));
            }
            return adminDao.updateAdmin(findAdmin);
        }else{
            //没有查询到,更新失败
            return -1;
        }
    }


    public Admin findAdmin(String adminName) {
        return adminDao.findAdminByAdminName(adminName);
    }


    public boolean login(String adminName, String plainPassword) throws IOException {
        Admin admin = adminDao.findAdminByAdminName(adminName);
        if (admin != null) {
            String adminPassword = admin.getPassword();
            if (StringUtils.hasText(plainPassword) && plainPassword.equals(new String(Base64Utils.decode(adminPassword),
                    Charset.defaultCharset()))) {
                return true;
            } else {
                //密码验证不成功
                return false;
            }
        }
        return false;
    }
}
