package com.adc.da.znks.service;

import com.adc.da.base.dao.BaseDao;
import com.adc.da.base.service.BaseService;
import com.adc.da.znks.dao.GatewayDao;
import com.adc.da.znks.entity.Gateway;
import com.adc.da.znks.util.SystemValue;
import io.netty.channel.Channel;
import org.apache.ibatis.annotations.Param;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * @author David
 */
@Service
@Transactional(value = "transactionManager", propagation = Propagation.REQUIRED, rollbackFor = Throwable.class)
public class GatewayService extends BaseService<Gateway, String> {

    private static final Logger logger = LoggerFactory.getLogger(GatewayService.class);

    @Autowired(required = false)
    private GatewayDao dao;

    @Override
    public BaseDao<Gateway> getDao() {
        return dao;
    }

    public Boolean getByPhoneNumber(String phoneNumber) {
        boolean flag = false;
        Gateway gateway = dao.getByPhoneNumber(phoneNumber);
        if(gateway != null) {
            flag = true;
        }
        return flag;
    }

    public void updateIpByMac(Gateway gateway){
        dao.updateIpByMac(gateway);
    }

    public List<Map<String,Object>> selectIpByMac(String ip){
        return dao.selectIp(ip);
    }

    /**
     * 用户修改网关mac地址（更换网关）
     * @param userId
     * @param macAddress
     * 1.根据userId清空用户关联的设备，记录等信息
     * 2.查看新的mac地址数据链路是否激活，激活状态的话就更新为用户的新网关
     */
    @Transactional
    public boolean updateMac(String userId,String macAddress){
        try{
            if(SystemValue.channelMap.get(macAddress)!=null){
                Channel ch = ((Channel) SystemValue.channelMap.get(macAddress));
                if(ch.isOpen() && ch.isActive()){
                    int result = dao.updateMacByUser(userId,macAddress);
                    if(result>0){
                        dao.deleteRecordByUser(userId);
                        dao.deleteDeviceByUser(userId);
                        return true;
                    }else{
                        return false;
                    }
                }
            }
        }catch(Exception e){
            e.printStackTrace();
        }
        return false;
    }

    public List<Gateway> checkGateWay(String userId,String macAddress){
        return dao.checkGateWay(userId,macAddress);
    }
}
