package com.adc.da.znks.service;

import com.adc.da.util.utils.DateUtils;
import com.adc.da.util.utils.GsonUtil;
import com.adc.da.znks.contants.ZnksConstants;
import com.adc.da.znks.dao.PositionDao;
import com.adc.da.znks.dao.UserDao;
import com.adc.da.znks.page.UserPage;
import com.adc.da.znks.util.DateTimeUtil;
import com.adc.da.znks.vo.UserCountVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;

/**
 * 统计用户service
 **/
@Service
public class UserCountService {

    @Autowired(required = false)
    private UserDao userDao;


    @Autowired(required = false)
    private PositionDao positionDao;

    private static final Logger logger = LoggerFactory.getLogger(UserCountService.class);


    /**
     * 统计用户VO的list
     **/
    private static List<UserCountVO> userCountVOS = new ArrayList<>(24);

    /**
     * 统计次数
     **/
    private static final int COUNT_SIZE = 24;


    /**
     * 获取用户统计信息vo list
     **/
    public List<UserCountVO> getUserCountInfo() {
        int countSize = UserCountService.COUNT_SIZE;
        for (int i = 0; i < COUNT_SIZE; i++) {
            Date recordTime1 = DateUtils.addHours(DateTimeUtil.getCurrentDayStartTime(),
                    i);
            userCountVOS.add(getUserCountInfo(recordTime1));
        }
        List<UserCountVO> userCountVOList = UserCountService.userCountVOS;
        logger.info("userCountVoList json: {}", GsonUtil.toJson(userCountVOList));
        return userCountVOList;
    }


    /**
     * 获取用户统计信息vo
     **/
    private UserCountVO getUserCountInfo(Date recordTime1) {
        UserCountVO userCountVO = new UserCountVO();
        userCountVO.setUserCount(getUserTotal());
        userCountVO.setClickRate(getClickRate());
        String recordTime1Str = DateUtils.dateToString(recordTime1, "yyyy-MM-dd HH:mm:ss");
        String recordTime2Str = getRecordTime2(recordTime1);
        userCountVO.setDistributePosition(getUserMainPosition(recordTime1Str, recordTime2Str));

        //获取每个时间段的人数
        userCountVO.setPerQuantumUserCount(String.valueOf(getPerQuantumUserCount(recordTime1Str, recordTime2Str)));
        logger.info("userCountVO: {}", userCountVO);
        return userCountVO;
    }


    /**
     * 获取每个时间段的人数
     **/
    private Integer getPerQuantumUserCount(String recordTime1, String recordTime2) {
        return positionDao.getUserCountByPerHour(recordTime1, recordTime2);
    }


    /**
     * 获取每个时间段的结束时间
     **/
    private String getRecordTime2(Date time1) {
        return DateUtils.dateToString(DateUtils.addHours(time1, 1),
                "yyyy-MM-dd HH:mm:ss");
    }


    /**
     * 从00:00-23:59,每一个小时统计一次,获取主要分布位置信息
     *
     * @param recordTime1 统计起始时间
     * @param recordTime2 统计结束时间
     **/
    private String getUserMainPosition(String recordTime1, String recordTime2) {
        TreeMap<Integer, String> mainPositionMap = new TreeMap<>();
        int positionAreaSize = ZnksConstants.TIANJIN_AREAS.length;
        List<String> positionAreas = CollectionUtils.arrayToList(ZnksConstants.TIANJIN_AREAS);
        for (int i = 0; i < positionAreaSize; i++) {
//            int userCount = positionDao.getUserCountInTianJinPerArea(positionAreas.get(i),
//                    recordTime1, recordTime2);
//            String positionArea = positionAreas.get(i);
//            mainPositionMap.put(userCount, positionArea);
        }
        String mainPosition = mainPositionMap.lastEntry().getValue();
        logger.info("mainPositionMap json: {}", GsonUtil.toJson(mainPositionMap));
        logger.info("main position: {},user count: {}", mainPositionMap.lastEntry().getValue(),
                mainPositionMap.lastEntry().getKey());
        return mainPosition;
    }

    /**
     * 获得软件总人数
     **/
    private int getUserTotal() {
        UserPage page = new UserPage();
        int userCount = userDao.queryByCount(page);
        return userCount;
    }


    /**
     * 获取每天客户使用软件的点击率
     **/
    private String getClickRate() {
        int currentDayPositionUserCount = positionDao.getCurrentDayUsers(new Date(System.currentTimeMillis()));
        double userCount = Double.parseDouble(String.valueOf(currentDayPositionUserCount));
        double userTotal = Double.parseDouble(String.valueOf(getUserTotal()));
        double clickRate = userCount / userTotal;
        String clickRateStr = String.valueOf(clickRate);
        clickRateStr = clickRateStr.substring(0, 5);
        logger.info("clickRate: {}", Double.parseDouble(clickRateStr));
        return clickRateStr;
    }

}
