package com.adc.da.znks.service;

import com.adc.da.base.service.BaseService;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.UUID;
import com.adc.da.znks.dao.PositionDao;
import com.adc.da.znks.dao.UserDao;
import com.adc.da.znks.entity.Position;
import com.adc.da.znks.entity.User;
import com.adc.da.znks.page.UserPage;
import com.adc.da.znks.util.SecurityUtils;
import com.adc.da.znks.vo.UserCountVO;
import com.adc.da.znks.vo.UserSearch;
import com.adc.da.znks.vo.UserVO;
import org.apache.http.util.Asserts;
import org.apache.ibatis.annotations.Param;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <br>
 * <b>功能：</b>UserService<br>
 *
 * @author ley
 */
@Service
@Transactional(value = "transactionManager", propagation = Propagation.REQUIRED, rollbackFor = Throwable.class)
public class UserService extends BaseService<User, String> {

    private static final Logger logger = LoggerFactory.getLogger(UserService.class);

    @Autowired(required = false)
    private UserDao dao;

    @Autowired(required = false)
    private PositionDao positionDao;

    @Override
    public UserDao getDao() {
        return dao;
    }

    public Boolean getUserByPhoneNumber(String phoneNumber) {
        Boolean flag = false;
        User user = dao.getUserByPhoneNumber(phoneNumber);
        if (user != null) {
            /*if(user.getPassword()!=null && !"".equals(user.getPassword())){
                flag = true;
            }else{
                flag = false;
            }*/
            flag = true;
        }
        return flag;
    }

    public User getByPhoneNumber(String phoneNumber) {

        return dao.getUserByPhoneNumber(phoneNumber);
    }

    /**
     * update password
     **/
    public boolean updatePassword(User user) {
        List<String> encryptList = encryptList(user.getPlainPassword());
        Asserts.notNull(encryptList, "encrypt list must not be null.");
        user.setEncryptkey(encryptList.get(0));
        user.setPassword(encryptList.get(1));
        UserPage userPage = new UserPage();
        user.setTemphone(user.getTemphone());
        User findUser = dao.queryByList(userPage).get(0);
        User updateUser = new User();
        updateUser.setPassword(encryptList.get(1));
        updateUser.setEncryptkey(encryptList.get(0));
        updateUser.setTemphone(user.getTemphone());
        int result = dao.updateByPrimaryKeySelective(updateUser);
        if (result != 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * user login or register
     **/
    public int loginOrRegister(String temphone, String plainPassword) {
        if (temphone == null || plainPassword == null || temphone.equals("") || plainPassword.equals("")) {
            return -1;
        }
        UserPage userPage = new UserPage();
        userPage.setTemphone(temphone);
        List<User> users = dao.queryByList(userPage);
        if (CollectionUtils.isEmpty(users)) {
            /*List<String> encryptList = encryptList(plainPassword);
            Asserts.notNull(encryptList, "encrypt list must not be null.");
            String id = UUID.randomUUID();
            User insertUser = new User(id, temphone, "", "");
            logger.info("insertUser: {}", insertUser);
            int result = dao.insertSelective(insertUser);
            if (result != 0) {
                return true;
            } else {
                return false;
            }*/
            return -1;
        } else {
            if (users.size() > 1) {
                return 0;
            }
            String mPassword = SecurityUtils.decrypt(users.get(0).getEncryptkey(), users.get(0).getPassword());
            if (plainPassword.equals(mPassword)) {
                return 1;
            }
            return 0;
        }
    }

    /**
     * user password encrypt list,list.get(0) is encryptKey,list.get(0) is encrypt password
     **/
    private List<String> encryptList(String plainPassword) {
        List<String> encryptList = new ArrayList<>(2);
        String encryptKey = UUID.randomUUID();
        encryptList.add(encryptKey);
        String encryptPassword = SecurityUtils.encrypt(plainPassword, encryptKey);
        encryptList.add(encryptPassword);
        logger.info("encryptList: {},encryptPassword: {}", encryptList, encryptPassword);
        return encryptList;
    }

    public List<Map<String, Object>> queryMacByPhone(String phone) {
        return dao.getGateWayByPhoneNumber(phone);
    }

    public void updateClientIdByPhoneNumber(String clientId, String phoneNumber) {
        dao.updateClientIdByPhoneNumber(clientId, phoneNumber);
    }

    public List<Map<String, Object>> queryClientByMac(String macAddress) {
        return dao.queryClientByMac(macAddress);
    }

    /**
     * 获取用户列表记录
     **/
    public List<UserVO> listUserVOsByCondition(UserSearch userSearch, Integer pageNUm, Integer pageSize) {
        List<UserVO> userVOS = dao.listUserVOs(userSearch, pageNUm, pageSize);
        return userVOS;
    }

    public boolean updateHomeName(User user) {
        if (dao.updateHomeName(user) > 0) {
            return true;
        }
        return false;
    }

    public String selectHomeNameByPhone(String temphone) {
        List<User> list = dao.selectHomeNameByPhone(temphone);
        if (!list.isEmpty()) {
            return list.get(0).getHomeName();
        }
        return null;
    }

    public boolean addUser(String temphone) {
        User user = dao.getUserByPhoneNumber(temphone);
        if (user != null) {
            return false;
        }
        String id = UUID.randomUUID();
        User insertUser = new User(id, temphone, "", "");
        logger.info("insertUser: {}", insertUser);
        int result = dao.insertSelective(insertUser);
        if (result > 0) {
            return true;
        }
        return false;
    }

    public int getUserCount() {
        UserPage userPage = new UserPage();
        return dao.queryByCount(userPage);
    }

    public int getCurrentDayUsers() {
        return positionDao.getCurrentDayUsers(new Date(System.currentTimeMillis()));
    }

    public int getUserCountInTianJinPerArea(String area) {
        return positionDao.getUserCountInTianJinPerArea(area);
    }

    public UserCountVO getUserCountVO() {
        UserPage userPage = new UserPage();
        int userCount = dao.queryByCount(userPage);
        UserCountVO userCountVO = new UserCountVO();
        userCountVO.setUserCount(userCount);
        double clickRate = positionDao.getCurrentDayUsers(new Date(System.currentTimeMillis())) / userCount;
        //userCountVO.setClickRate(clickRate);
        //userCountVO.setDistributeVOS(ZnksTaskService.getDistributeVOS());
        return userCountVO;
    }
}
